/***
 * AOKell
 * Copyright (C) 2006 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.aokell.glue.template;

import org.objectweb.fractal.aokell.lib.control.component.ComponentItf;
import org.objectweb.fractal.aokell.lib.control.component.ComponentSetterItf;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;

import spoon.template.Local;
import spoon.template.Template;

/**
 * This template defines the elements which are introduced in all classes
 * which are processed by this ComponentControllerProcessor.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class ComponentControllerTemplate implements Template, Component, ComponentSetterItf {
    
    @Local
    public ComponentControllerTemplate() {}
    
    /**
     * The reference towards the component controller.
     * 
     * Visibility set to protected to let the field be visible by subclasses.
     */
    protected ComponentItf _fcComp;
    
    
    // -----------------------------------------------------------
    // Component interface implementation
    // -----------------------------------------------------------
    
    public Type getFcType() {
        return _fcComp.getFcType();
    }

    public Object getFcInterface(String interfaceName)
    throws NoSuchInterfaceException {
        return _fcComp.getFcInterface(interfaceName);
    }
    
    public Object[] getFcInterfaces() {
        return _fcComp.getFcInterfaces();
    }

    
    // ------------------------------------------------------------------
    // ComponentSetterItf interface implementation
    // ------------------------------------------------------------------

    public void setFcCompCtrl( ComponentItf _fcComp ) {
        this._fcComp = _fcComp;
    }    
}
